/*
 * Copyright 04/26/95 Sun Microsystems, Inc.  All Rights Reserved.
 */
 
/*
 * sys/note.h: interface for annotating source with info for lock_lint.
 *
 * This is the underlying interface; NOTE (include/cc/ssbd/note.h) is the
 * preferred interface, but all exported header files should include this
 * file directly and use _NOTE so as not to take "NOTE" from the user's
 * namespace.  For consistency, *all* kernel source should use _NOTE.
 *
 * By default, annotations expand to nothing.  This file is the
 * interposed version which expands annotations for lock_lint.
 *
 * The _NOTE macro in /usr/include/sys/note.h expands to nothing,
 * so that compilers and other tools that are not NOTE-savvy don't
 * even see them.  But this file gets included instead when using the
 * -Zll flag.  It causes the annotations to be converted into constructs
 * which the grammar of the compiler used with the -Zll flag is prepared
 * to see.
 *
 * There are two translation schemes.  The first is represented by
 * __ANNOTATION1 and __ANNOTATION2, and is used in cases where you
 * have a specified number of non-expression arguments which are to
 * be parsed by the tool itself.  For example, MUTEX_PROTECTS_DATA
 * accepts a list of variables which involves some very non-C syntax.
 * We just make a string out of each argument and hand it to the tool.
 *
 * The other style supports an arbitrary number of C expression
 * arguments.  To get this varargs effect out of a macro we define
 * the macro without arguments, and let the argument list remain
 * as a parenthesized comma expression.  It appears to the parser
 * as a single expression (a comma expression), and the tool can
 * then pull out the individul arguments as it needs them.  Annotation
 * ASSUMING_PROTECTED is of this class.
 *
 * Of course, some annotations don't take any arguments at all.
 * Note that other styles of handling expressions are possible, but
 * these are sufficient to handle all current lock_lint NOTEs.
 */
 
#ifndef _SYS_NOTE_H
#define _SYS_NOTE_H

#pragma ident "@(#)note.h 1.2 95/04/26 SMI"

#define _NOTE(note)	___note(__NOTE__ ## note)___note

#define __ANNOTATION1(name,str1)	#name(#str1)
#define __ANNOTATION2(name,str1,str2)	#name(#str1,#str2)

/*
 * e.g,    __NOTE(MUTEX_PROTECTS_DATA(m, x t::{a b}))
 * becomes ___note(__NOTE__MUTEX_PROTECTS_DATA(m, x t::{a b}))___note
 * becomes ___note(__ANNOTATION2(MUTEX_PROTECTS_DATA, m, x t::{a b}))___note
 * becomes ___note("MUTEX_PROTECTS_DATA"( "m", "x t::{a b}"))___note
 *
 * e.g.    __NOTE(ASSUMING_PROTECTED(*p, *q))
 * becomes ___note(__NOTE__ASSUMING_PROTECTED(*p, *q))___note
 * becomes ___note("ASSUMING_PROTECTED"(*p, *q))___note
 */

#define __NOTE__SCHEME_PROTECTS_DATA(scheme,data)		__ANNOTATION2(SCHEME_PROTECTS_DATA,scheme,data)
#define __NOTE__MUTEX_PROTECTS_DATA(mutex,data)			__ANNOTATION2 (MUTEX_PROTECTS_DATA,mutex, data)
#define __NOTE__RWLOCK_PROTECTS_DATA(rwlock,data)		__ANNOTATION2(RWLOCK_PROTECTS_DATA,rwlock,data)

#define __NOTE__READ_ONLY_DATA(data)				__ANNOTATION1(READ_ONLY_DATA,data)
#define __NOTE__DATA_READABLE_WITHOUT_LOCK(data)		__ANNOTATION1(DATA_READABLE_WITHOUT_LOCK,data)
#define __NOTE__RWLOCK_COVERS_LOCKS(cover_lock,covered_locks) 	__ANNOTATION2(RWLOCK_COVERS_LOCKS,cover_lock,covered_locks)
#define __NOTE__LOCK_ORDER(locks)				__ANNOTATION1(LOCK_ORDER,locks)

#define __NOTE__MUTEX_ACQUIRED_AS_SIDE_EFFECT(mutex)		"MUTEX_ACQUIRED_AS_SIDE_EFFECT"(mutex)
#define __NOTE__READ_LOCK_ACQUIRED_AS_SIDE_EFFECT(rwlock)	"READ_LOCK_ACQUIRED_AS_SIDE_EFFECT"(rwlock)
#define __NOTE__WRITE_LOCK_ACQUIRED_AS_SIDE_EFFECT(rwlock)	"WRITE_LOCK_ACQUIRED_AS_SIDE_EFFECT"(rwlock)
#define __NOTE__LOCK_RELEASED_AS_SIDE_EFFECT(lock)		"LOCK_RELEASED_AS_SIDE_EFFECT"(lock)
#define __NOTE__LOCK_UPGRADED_AS_SIDE_EFFECT(rwlock)		"LOCK_UPGRADED_AS_SIDE_EFFECT"(rwlock)
#define __NOTE__LOCK_DOWNGRADED_AS_SIDE_EFFECT(rwlock)		"LOCK_DOWNGRADED_AS_SIDE_EFFECT"(rwlock)

#define __NOTE__NOW_INVISIBLE_TO_OTHER_THREADS			"NOW_INVISIBLE_TO_OTHER_THREADS"
#define __NOTE__NOW_VISIBLE_TO_OTHER_THREADS			"NOW_VISIBLE_TO_OTHER_THREADS"
#define __NOTE__ASSUMING_PROTECTED				"ASSUMING_PROTECTED"

#define __NOTE__NO_COMPETING_THREADS_AS_SIDE_EFFECT		"NO_COMPETING_THREADS_AS_SIDE_EFFECT"
#define __NOTE__COMPETING_THREADS_AS_SIDE_EFFECT		"COMPETING_THREADS_AS_SIDE_EFFECT"
#define __NOTE__COMPETING_THREADS_NOW				"COMPETING_THREADS_NOW"
#define __NOTE__NO_COMPETING_THREADS_NOW			"NO_COMPETING_THREADS_NOW"

#define __NOTE__NOT_REACHED					"NOT_REACHED"

#endif  /* _SYS_NOTE_H */
